"""
Solves the continuum intracellular-transport problem using default model parameters for vimentin
"""

from dolfin import *
import numpy as np
from mshr import * 

ffc_options = {"quadrature_degree": 4} # set quadrature degree for the solver
                                            
# Model parameters

HalfSize = 2.5  # micron; half of the domain size
D = 2*HalfSize # micron; domain size
BeadRadius = 0.25  # micron; radius of the bead
PullAngle = pi/6 # pulling angle
Y = 0.9*pow(10,6) # nN per micron^2; Young's modulus of vimentin
b = 5.0*pow(10,-3) # micron; radius of vimentin filament
TensileStrength = 8 # nN; tensile strength of vimentin
NC = 100 # N representative of cytoskeleton
RC = D/NC # micron; mesh size at t=0

# Define domain, generate mesh with a given resolution and define appropriate function spaces

res = 400
ret = Rectangle(Point(-HalfSize,-2.5), Point(HalfSize,2.5))
circ = Circle(Point(0.0,0.0),BeadRadius)
domain = ret - circ
mesh = generate_mesh(domain, res)
n = FacetNormal(mesh)
VScalar = FunctionSpace(mesh, 'P', 1)
VVector = VectorFunctionSpace(mesh, 'P', 1)
VTensor = TensorFunctionSpace(mesh,'P',1)

# Define boundary parts and boundary conditions

boundary_parts = \
MeshFunction("size_t", mesh, mesh.topology().dim()-1)
boundary_parts.set_all(1)

class BottomDirichletBoundary(SubDomain):
	def inside(self, x, on_boundary):
		tol = 0.0000001 # tolerance for coordinate comparisons
		return on_boundary and abs(x[1]+HalfSize) < tol
BottomDirichletBoundary().mark(boundary_parts, 2)

class RightDirichletBoundary(SubDomain):
	def inside(self, x, on_boundary):
		tol = 0.0000001 # tolerance for coordinate comparisons
		return on_boundary and abs(x[0]-HalfSize) < tol
RightDirichletBoundary().mark(boundary_parts, 3)

class TopDirichletBoundary(SubDomain):
	def inside(self, x, on_boundary):
		tol = 0.0000001 # tolerance for coordinate comparisons
		return on_boundary and abs(x[1]-HalfSize) < tol
TopDirichletBoundary().mark(boundary_parts, 4)

class LeftDirichletBoundary(SubDomain):
	def inside(self, x, on_boundary):
		tol = 0.0000001 # tolerance for coordinate comparisons
		return on_boundary and abs(x[0]+HalfSize) < tol
LeftDirichletBoundary().mark(boundary_parts, 5)

class CentralDirichletBoundary(SubDomain):
	def inside(self, x, on_boundary):
		tol = 0.01 # tolerance for coordinate comparisons
		return on_boundary and abs((x[0]*x[0]+x[1]*x[1])-BeadRadius*BeadRadius) < tol
CentralDirichletBoundary().mark(boundary_parts, 6)

uBoundary = 0.0 # micron; outer boundary condition (X component)
vBoundary = 0.0 # micron; outer boundary condition (Y component)
Gamma_Top = DirichletBC(VVector,(uBoundary,vBoundary),TopDirichletBoundary())
Gamma_Bottom = DirichletBC(VVector,(uBoundary,vBoundary),BottomDirichletBoundary())
Gamma_Left = DirichletBC(VVector,(uBoundary,vBoundary),LeftDirichletBoundary())
Gamma_Right = DirichletBC(VVector,(uBoundary,vBoundary),RightDirichletBoundary())
ds = ds(domain=mesh, subdomain_data= boundary_parts)

# Define unit vectors in X and Y directions

UnitE1 = Expression(("1","0"),degree=1)
UnitE2 = Expression(("0","1"),degree=1)

# Define a smooth approximation to the strain energy, modulated by parameter LargeKappa as presented in Section S4.3 of Supplementary material

LargeKappa = 200

def EVim(y):
    return ksVim/2*pow((y-1/xiVim)/(1+exp(-LargeKappa*(y-1/xiVim))),2)

for fpMacro in [0.5*TensileStrength]: # nN, macroscale dimensional prestress; equal to one half of the tensile strength

 # define parameters of the simplified (linearized) approximation for vimentin
  
 fpMacroDimless = fpMacro/(pi*Y*b*b) # dimless prestress
 xiVim = 1+ fpMacroDimless # xi for vimentin
 ksVim = pi*Y*b*b*xiVim/RC # nN per micron; dimensional spring constant
 
 # create output files for displacement vector field and force-displacement text file
 
 fileVector = File("DisplVectorField.pvd")
 fileForceDispl = open("ForceDispl.txt","w")
 
 # Solve the continuum problem for increasing bead displacement (magnitude)
 
 for NormCentralDisplacement in [0.0, 0.005, 0.01, 0.015, 0.02, 0.025, 0.03, 0.035, 0.04, 0.045, 0.05]: #micron
 
  uCentral = NormCentralDisplacement*cos(PullAngle)
  vCentral = NormCentralDisplacement*sin(PullAngle)
  Gamma_Central = DirichletBC(VVector,(uCentral,vCentral),CentralDirichletBoundary())
  bcs = [Gamma_Top,Gamma_Bottom,Gamma_Left,Gamma_Right, Gamma_Central]
  
  # Define functions
  
  du = TrialFunction(VVector)            # Incremental displacement
  v  = TestFunction(VVector)             # Test function
  u  = Function(VVector)                 # Displacement from previous iteration
  
  # Kinematics
  
  d = u.geometric_dimension()
  I = Identity(d)             # Identity tensor
  F = I + grad(u)             # Deformation gradient
  C = F.T*F                   # Right Cauchy-Green tensor
  
  psi = EVim(sqrt(dot(UnitE1,dot(C,UnitE1)))) + EVim(sqrt(dot(UnitE2,dot(C,UnitE2)))) # Stored strain energy density
  Pi = psi*dx # Total energy
  FF = derivative(Pi, u, v) # Compute first variation of Pi (directional derivative about u in the direction of v)
  J = derivative(FF, u, du)  # Compute Jacobian of FF
  solve(FF == 0, u, bcs, J=J, form_compiler_parameters=ffc_options, solver_parameters={"newton_solver":{"maximum_iterations":500,'linear_solver':'mumps'}}) # Solve variational problem  
  fileVector << (u,NormCentralDisplacement) # export the displacement vector field into the output file
  
  # calculate stress tensor field
  uX = project(dot(UnitE1,u),VScalar)
  uY = project(dot(UnitE2,u),VScalar)
  TensorNominalStress = project(ksVim*as_tensor([[(1+nabla_grad(uX)[0])*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uX)[0],2)+pow(nabla_grad(uY)[0],2))))/(1+exp(-LargeKappa*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uX)[0],2)+pow(nabla_grad(uY)[0],2)))))),nabla_grad(uY)[0]*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uX)[0],2)+pow(nabla_grad(uY)[0],2))))/(1+exp(-LargeKappa*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uX)[0],2)+pow(nabla_grad(uY)[0],2))))))],[nabla_grad(uX)[1]*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uY)[1],2)+pow(nabla_grad(uX)[1],2))))/(1+exp(-LargeKappa*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uY)[1],2)+pow(nabla_grad(uX)[1],2)))))),(1+nabla_grad(uY)[1])*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uY)[1],2)+pow(nabla_grad(uX)[1],2))))/(1+exp(-LargeKappa*(1-1/(xiVim*sqrt(pow(1+nabla_grad(uY)[1],2)+pow(nabla_grad(uX)[1],2))))))]]),VTensor)
  
  # calculate the net force on the bead (X- and Y- components as well as its magnitude) and write results into force-displacement file
  
  FFx = assemble(-dot(TensorNominalStress.T,n)[0]*ds(6))
  FFy = assemble(-dot(TensorNominalStress.T,n)[1]*ds(6))
  print(FFx,FFy,sqrt(FFx*FFx+FFy*FFy))
  fileForceDispl.write(str(NormCentralDisplacement)+ " " + str(FFx)+ " " + str(FFy) + " " + str(sqrt(FFx*FFx+FFy*FFy)) + "\n")
 fileForceDispl.close()
