clear all
clc
close all

Y = 0.9*10^6; % nN per micron^2; Young's modulus of vimentin
b = 5.0*10^(-3); % micron; radius of vimentin filament
phi = pi/6; % pulling angle
D = 5; % micron; domain size
NCS = 100; % N representative of cytoskeleton
BeadDiameter=0.5;  %micron; bead diameter
BeadDisplStep=0.005; % micron; step in bead displacement
TensileStrength = 8; % nN; tensile strength
fpMacro = TensileStrength/2; % nN; dimensional force due to prestress; equal to one half of the tensile strength
fpMacroDimless = fpMacro/(pi*Y*b*b); % dimensionless force due to prestress
Ks = NCS*pi*Y*b*b/D % nN per micron; dimensional spring constant (linearized model)

for N=[100] % number of filament segments per filament - decrease for faster simulations

delta = D/N; % micron; mesh spacing at t=0

% the initial network (which is also the initial guess for the solver)

for ii=1:(N+1)
    for jj=1:(N+1)
                x0(jj,ii,1)= delta*(ii-(N+2)/2); % X-coordinates
                x0(jj,ii,2)= delta*(jj-(N+2)/2); % Y-coordinates
    end
end

exitflag = 1; % boolean variable checking for success of the solution process
l=0; %index that increases with increasing bead displacement

while (exitflag>0 & l<11)
    Rb=BeadDisplStep*l % display the current bead displacement
    [newArrayX(l+1),newArrayY(l+1),exitflag,x0]=SolveAtGivenBeadDispl(N,Rb,delta,phi,x0,BeadDiameter,fpMacroDimless,Ks); % solve the discrete problem at given bead displacement and output the X and Y components of the net force, the exitflag and the solution (node coordinates)
    l=l+1; % increase the bead displacement
end
end

TotalTensileForceMagnitudeArray = sqrt(newArrayX.^2 + newArrayY.^2); % vector with magnitude of the net force for increasing bead displacement
AuxVector = 0:BeadDisplStep:(length(TotalTensileForceMagnitudeArray)-1)*BeadDisplStep; % vector of (the magnitudes of) bead displacements
ForceDisplMatrix = [AuxVector' newArrayX' newArrayY' TotalTensileForceMagnitudeArray']
csvwrite(strcat('ForceDispl.csv'),ForceDisplMatrix) % export into a .csv file a matrix with bead displacements, X- and Y-components and magnitudes of the net force as columns


function [FxBead,FyBead,exitflag,x0]=SolveAtGivenBeadDispl(N,Rb,delta,phi,x0,BeadDiameter,fpMacroDimless,Ks)

% solve the discrete problem and export the solution

fun = @root2d;
options = optimoptions('fsolve'); 
[x,fval,exitflag,output] = fsolve(fun,x0,options);

exitflag % display the success of the solution process

% Sum up forces from all filament segments (X and Y components) attached to the bead

FxBead = 0;
FyBead = 0;

for i = (round((N+2)/2-BeadDiameter/(2*delta))-1):(round((N+2)/2+BeadDiameter/(2*delta))+1)
    for j = (round((N+2)/2-BeadDiameter/(2*delta))-1):(round((N+2)/2+BeadDiameter/(2*delta))+1)
        if delta*sqrt((i-(N+2)/2)^2+(j-(N+2)/2)^2)<=BeadDiameter/2
            if delta*sqrt((i-1-(N+2)/2)^2+(j-(N+2)/2)^2)>BeadDiameter/2 %left neighbour
                [FMagnitudeBeadLeft,BeadFLeftX,BeadFLeftY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j,i-1,1),x(j,i-1,2));
                FxBead = FxBead + BeadFLeftX;
                FyBead = FyBead + BeadFLeftY;
            end
            
            if delta*sqrt((i+1-(N+2)/2)^2+(j-(N+2)/2)^2)>BeadDiameter/2 % right neighbour
                [FMagnitudeBeadRight,BeadFRightX,BeadFRightY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j,i+1,1),x(j,i+1,2));
                FxBead = FxBead + BeadFRightX;
                FyBead = FyBead + BeadFRightY;
            end
            
            if delta*sqrt((i-(N+2)/2)^2+(j-1-(N+2)/2)^2)>BeadDiameter/2 % bottom neighbour
                [FMagnitudeBeadBottom,BeadFBottomX,BeadFBottomY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j-1,i,1),x(j-1,i,2));
                FxBead = FxBead + BeadFBottomX;
                FyBead = FyBead + BeadFBottomY;
            end

            if delta*sqrt((i-(N+2)/2)^2+(j+1-(N+2)/2)^2)>BeadDiameter/2 % top neighbour
                [FMagnitudeBeadTop,BeadFTopX,BeadFTopY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j+1,i,1),x(j+1,i,2));
                FxBead = FxBead + BeadFTopX;
                FyBead = FyBead + BeadFTopY;
            end
            
        end
    end
end

% If solution process was successful, export X and Y components of the solution

if exitflag>0
csvwrite(strcat('XComponentsDisplacement',sprintf('%.3f',Rb),'.csv'),x(:,:,1))
csvwrite(strcat('YComponentsDisplacement',sprintf('%.3f',Rb),'.csv'),x(:,:,2))
end

% Update the initial guess to be the latest solution

for ii=1:(N+1)
    for jj=1:(N+1)
           x0(jj,ii,1)= x(jj,ii,1);
           x0(jj,ii,2)= x(jj,ii,2);       
    end
end


% Function below defines a large vector F of functions dependent on the node coordinates x(:,:,1) and
% x(:,:,2). The main code then solves a large algebraic system of equations
% F(x)=0 for every value of bead displacement (magnitude)

function F = root2d(x)

for i=1:(N+1)
    for j=1:(N+1)
        
        % Fixed displacement for the bead
        
        if ( delta*sqrt((i-(N+2)/2)^2+(j-(N+2)/2)^2)<=BeadDiameter/2)
            F(2*((N+1)*(j-1)+i)-1)= x(j,i,1)-Rb*cos(phi)- delta*(i-(N+2)/2);
            F(2*((N+1)*(j-1)+i))= x(j,i,2)-Rb*sin(phi)- delta*(j-(N+2)/2);
        
        % Zero displacement for the left boundary
        
        elseif i==1
            F(2*((N+1)*(j-1)+i)-1)= x(j,i,1)+ delta*N/2;
            F(2*((N+1)*(j-1)+i))= x(j,i,2)- delta*(j-(N+2)/2);
            
        % Zero displacement for the right boundary
                    
        elseif i== (N+1)
            F(2*((N+1)*(j-1)+i)-1)= x(j,i,1)- delta*N/2;
            F(2*((N+1)*(j-1)+i))= x(j,i,2)- delta*(j-(N+2)/2);
            
        % Zero displacement for the bottom boundary
        
        elseif j==1
            F(2*((N+1)*(j-1)+i)-1)= x(j,i,1)- delta*(i-(N+2)/2);
            F(2*((N+1)*(j-1)+i))= x(j,i,2)+ delta*N/2;
        
        % Zero displacement for the top boundary
                
        elseif j==(N+1)
            F(2*((N+1)*(j-1)+i)-1)= x(j,i,1)- delta*(i-(N+2)/2);
            F(2*((N+1)*(j-1)+i))= x(j,i,2)- delta*N/2;
            
        % Otherwise, net force = 0 for internal nodes
        
        else
            [FMagnitudeRight,FRightX,FRightY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j,i+1,1),x(j,i+1,2));
            [FMagnitudeLeft,FLeftX,FLeftY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j,i-1,1),x(j,i-1,2));
            [FMagnitudeTop,FTopX,FTopY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j+1,i,1),x(j+1,i,2));
            [FMagnitudeBottom,FBottomX,FBottomY]= FilamentForceLinearOnlyTensile(x(j,i,1),x(j,i,2),x(j-1,i,1),x(j-1,i,2));                                
            F(2*((N+1)*(j-1)+i)-1) = (FRightX+FLeftX+FTopX+FBottomX);
            F(2*((N+1)*(j-1)+i)) = (FRightY+FLeftY+FTopY+FBottomY);     
                     
        end
    end
end
end

% Function below outputs axial force that filament segment experiences (magnitude, X and Y components) if its two end points are at (x1,y1) and (x2,y2)

function [FMagnitude,Fx,Fy]= FilamentForceLinearOnlyTensile(x1,y1,x2,y2) 

    FMagnitude = max(0,Ks*((1+fpMacroDimless)*sqrt((x2-x1)^2+(y2-y1)^2)-delta)); 
    Fx = FMagnitude*(x2-x1)/sqrt((x2-x1)^2+(y2-y1)^2);
    Fy = FMagnitude*(y2-y1)/sqrt((x2-x1)^2+(y2-y1)^2);
    
end

end
